#include <stdlib.h>
#include <stdio.h>
#include <GL/glut.h>
#include <GL/gl.h>
#include "simage.h"
#include "cube.h"

namespace oglclass
{

cube::cube(float w, float h, float l)
{
	type_ = CUBE_WIREFRAME;
	width_ = w; height_ = h; length_ = l;
	textureIndex_ = 0;
	makeCube();
}

cube::~cube()
{
}

void cube::makeCube()
{
    vertex_[0][0] = width_/2; vertex_[0][1] = height_/2; vertex_[0][2] = length_/2; 
    vertex_[1][0] = width_/2; vertex_[1][1] = -height_/2; vertex_[1][2] = length_/2; 
    vertex_[2][0] = width_/2; vertex_[2][1] = -height_/2; vertex_[2][2] = -length_/2; 
    vertex_[3][0] = width_/2; vertex_[3][1] = height_/2; vertex_[3][2] = -length_/2; 
    vertex_[4][0] =  -width_/2; vertex_[4][1] = height_/2; vertex_[4][2] = length_/2; 
    vertex_[5][0] =  -width_/2; vertex_[5][1] = -height_/2; vertex_[5][2] = length_/2; 
    vertex_[6][0] =  -width_/2; vertex_[6][1] = -height_/2; vertex_[6][2] = -length_/2; 
    vertex_[7][0] =  -width_/2; vertex_[7][1] = height_/2; vertex_[7][2] = -length_/2; 

    index_[0][0] = 0;index_[0][1] = 1;index_[0][2] = 2;index_[0][3] = 3;
    index_[1][0] = 0;index_[1][1] = 4;index_[1][2] = 5;index_[1][3] = 1;
    index_[2][0] = 4;index_[2][1] = 7;index_[2][2] = 6;index_[2][3] = 5;
    index_[3][0] = 7;index_[3][1] = 3;index_[3][2] = 2;index_[3][3] = 6;
    index_[4][0] = 0;index_[4][1] = 3;index_[4][2] = 7;index_[4][3] = 4;
    index_[5][0] = 1;index_[5][1] = 5;index_[5][2] = 6;index_[5][3] = 2;

	normal_[0][0] =  1.0; normal_[0][1] =  0.0; normal_[0][2] =  0.0;
	normal_[1][0] =  0.0; normal_[1][1] =  1.0; normal_[1][2] =  0.0;
	normal_[2][0] =  0.0; normal_[2][1] =  0.0; normal_[2][2] =  1.0;
	normal_[3][0] = -1.0; normal_[3][1] =  0.0; normal_[3][2] =  0.0;
	normal_[4][0] =  0.0; normal_[4][1] = -1.0; normal_[4][2] =  0.0;
	normal_[5][0] =  0.0; normal_[5][1] =  0.0; normal_[5][2] = -1.0;

	texture_[0][0] = 1.0; texture_[0][1] = 1.0; 
	texture_[1][0] = 0.0; texture_[1][1] = 1.0; 
	texture_[2][0] = 0.0; texture_[2][1] = 0.0; 
	texture_[3][0] = 1.0; texture_[3][1] = 0.0; 
}

void cube::draw()
{
    switch(type_)
    {
    case CUBE_WIREFRAME:
        drawWireframe();
        break;
    case CUBE_SOLID:
        drawSolid();
        break;
    case CUBE_TRANSPARENT:
        drawTransparent();
        break;
    case CUBE_TEXTURE:
        drawTexture();
        break;
	case CUBE_TEXTURE_TRANSPARENT:
        drawTextureTransparent();
        break;
    default:
        break;
    }
}

void cube::drawWireframe()
{
	for (int i=0; i<6; i++) {
		glBegin(GL_LINE_LOOP);
	    glNormal3fv(normal_[i]);
	    for(int j=0; j<4; j++) {
		glVertex3fv(vertex_[index_[i][j]]);
	    }
		glEnd();
	}

}

void cube::drawSolid()
{
	for (int i=0; i<6; i++) {
		glBegin(GL_POLYGON);
	    glNormal3fv(normal_[i]);
	    for(int j=0; j<4; j++) {
		glVertex3fv(vertex_[index_[i][j]]);
	    }
		glEnd();
	}
}

void cube::drawTransparent()
{
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 

	for (int i=0; i<6; i++) {
	    glBegin(GL_POLYGON);	      
	    glNormal3fv(normal_[i]);
	    for(int j=0; j<4; j++) {
		glVertex3fv(vertex_[index_[i][j]]);
	    }
	    glEnd();
	}

	glDisable(GL_BLEND);
}

void cube::drawTexture()
{
	// texture on
	glEnable(GL_TEXTURE_2D);
	glCallList(textureIndex_); 

    glBindTexture(GL_TEXTURE_2D, textureIndex_);
	for(int i=0; i<6; i++) {
	    glBegin(GL_POLYGON);	      
	    glNormal3fv(normal_[i]);
	    for(int j=0; j<4; j++) {
		glTexCoord2fv(texture_[j]);
		glVertex3fv(vertex_[index_[i][j]]);
	    }
	    glEnd();
	}

    glBindTexture(GL_TEXTURE_2D, 0);
    glDisable(GL_TEXTURE_2D);
}

void cube::drawTextureTransparent()
{
	// blending on
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 

	// texture on
	glEnable(GL_TEXTURE_2D);
	glCallList(textureIndex_); 

    glBindTexture(GL_TEXTURE_2D, textureIndex_);
	for(int i=0; i<6; i++) {
	    glBegin(GL_POLYGON);	      
	    glNormal3fv(normal_[i]);
	    for(int j=0; j<4; j++) {
		glTexCoord2fv(texture_[j]);
		glVertex3fv(vertex_[index_[i][j]]);
	    }
	    glEnd();
	}

	// texture off
    glBindTexture(GL_TEXTURE_2D, 0);
    glDisable(GL_TEXTURE_2D);

	// blending off
	glDisable(GL_BLEND);
}

void cube::initTexture(char *filename, GLfloat wrap)
{
	unsigned char *imageBuffer, *rescaledImage, *imagePtr;
	int imageWidth = 0, imageHeight = 0, numComponents = 0;
	imageBuffer = simage_read_image(filename, &imageWidth, &imageHeight, &numComponents);

	GLsizei xdim2,ydim2;

	xdim2 = 1;
    while (xdim2 <= imageWidth)
        xdim2 *= 2;
    xdim2 /= 2;
    ydim2 = 1;
    while (ydim2 <= imageHeight)
        ydim2 *= 2;
    ydim2 /= 2;
    if ((imageWidth != xdim2) || (imageHeight != ydim2))
        {
        rescaledImage = simage_resize(imageBuffer, imageWidth, imageHeight, numComponents,
                                           xdim2, ydim2);
        imagePtr = rescaledImage;
        }
    else
        imagePtr = imageBuffer;

	if (imagePtr != NULL)
	{
		glGenTextures(1, &textureIndex_);	// set textureIndex_
		glBindTexture(GL_TEXTURE_2D, textureIndex_);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, numComponents == 3 ? GL_RGB : GL_RGBA, imageWidth, imageHeight,
					 0, numComponents == 3 ? GL_RGB : GL_RGBA, GL_UNSIGNED_BYTE, imageBuffer);
		glBindTexture(GL_TEXTURE_2D, 0);

		printf("texturefile=%s textureIndex_=%d\n", filename, textureIndex_);
	}


}

}