#include <windows.h>
#include <stdio.h>
#include <gl/glut.h>
#include "cube.h"

using namespace oglclass;
cube box(2, 2, 2);

float g_viewDistance = 5.0;
float g_viewRotY = 0.0;
float g_viewRotX = 0.0;
float g_viewCameraX = 0.0;
float g_viewCameraY = 0.0;
float g_prevTime = 0.0;

GLuint	g_fogMode[]= { GL_EXP, GL_EXP2, GL_LINEAR };    // Storage For Three Types Of Fog (NEW)
GLuint	g_fogfilter = 0;                                // Which Fog Mode To Use (NEW)

void initLight(void)
{
	GLfloat lightAmbient[]  =	{ 0.5f, 0.5f, 0.5f, 1.0f }; 
	GLfloat lightDiffuse[]  =	{ 1.0f, 1.0f, 1.0f, 1.0f }; 
	GLfloat lightPosition[] = { 0.0f, 0.0f, 2.0f, 1.0f };
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
}

void initFog()
{
	GLfloat	fogColor[4] = {0.5f,0.5f,0.5f,1.0f};          // Fog Color (NEW)
	glFogi(GL_FOG_MODE, g_fogMode[g_fogfilter]);       // Fog Mode
	glFogfv(GL_FOG_COLOR, fogColor);                 // Set Fog Color
	glFogf(GL_FOG_DENSITY, 0.35f);                     // How Dense Will The Fog Be
	glHint(GL_FOG_HINT, GL_DONT_CARE);                 // Fog Hint Value
	glFogf(GL_FOG_START, 1.0f);                        // Fog Start Depth
	glFogf(GL_FOG_END, 5.0f);                          // Fog End Depth
	glEnable(GL_FOG);                                  // Enables GL_FOG
}

void init(void)
{
    glShadeModel(GL_SMOOTH);						   // Enable Smooth Shading
	glClearColor(0.5f,0.5f,0.5f,1.0f);                 // We'll Clear To The Color Of The Fog (CHANGED)
    glClearDepth(1.0f);								   // Depth Buffer Setup
    glEnable(GL_DEPTH_TEST);						   // Enables Depth Testing
    glDepthFunc(GL_LEQUAL);							   // The Type Of Depth Testing To Do
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);             // Pixel Storage Mode To Byte Alignment
    glEnable(GL_CULL_FACE);                            // Cull Polygons
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	initLight();
	initFog();

	box.initTexture("opengl.jpg");
	box.setDrawType(CUBE_TEXTURE_TRANSPARENT);
}

void draw(void)   
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer
	glLoadIdentity();									// Reset The View
	
	glTranslatef(0.0, 0.0, -g_viewDistance);
    glRotatef(g_viewRotX, 1.0, 0.0, 0.0);
    glRotatef(g_viewRotY, 0.0, 1.0, 0.0);

	box.draw();

    glutSwapBuffers ( );
}

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);     // Select The Projection Matrix
	glLoadIdentity();                // Reset The Projection Matrix
	gluPerspective(60, (float)w/(float)h, 1.0, 1000.0);
	glMatrixMode(GL_MODELVIEW);      // Select The Modelview Matrix
	glLoadIdentity(); // Reset The Modelview Matrix
}

void keyboard(unsigned char key, int x, int y)
{
	switch (key) {
		case 27:        
			exit(0);    
			break;
		case ' ':
			g_fogfilter += 1;
			if (g_fogfilter > 2) g_fogfilter = 0;
			printf("fogfilter=%d\n", g_fogfilter);
			glFogi(GL_FOG_MODE, g_fogMode[g_fogfilter]); // Fog Mode
			break;
		case '+':
			g_viewDistance += 0.5;
		case '-':
			g_viewDistance -= 0.5;
		default:
			break;
	}
}

void specialkey(int key, int x, int y)
{
    if (key == GLUT_KEY_LEFT)
        g_viewRotY += 3;
    else if (key == GLUT_KEY_RIGHT)
        g_viewRotY -= 3;
    else if (key == GLUT_KEY_UP)
        g_viewRotX += 3;
    else if (key == GLUT_KEY_DOWN)
        g_viewRotX -= 3;
    glutPostRedisplay();
}

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE); 
	glutInitWindowSize(500, 500);                
	glutCreateWindow("Fog"); 
	init();
	glutDisplayFunc(draw);                        
	glutReshapeFunc(reshape);                      
	glutKeyboardFunc(keyboard);                      
    glutSpecialFunc(specialkey);
	glutMainLoop();                                  
	return 0;
}
